/*
*  gnupg.h 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#ifndef gnupgH
#define gnupgH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <syncobjs.hpp>
#include <Menus.hpp>
class TStdIOInputThread;
class TStdIOOutputThread;
//---------------------------------------------------------------------------
bool __fastcall GPGProcessMessageDecryption(TForm *Owner, TStrings *Message);

AnsiString __fastcall GPGExe();
AnsiString __fastcall GPGDir();
AnsiString __fastcall GPGDataDir();
AnsiString __fastcall GPGConf();
AnsiString __fastcall GPGIconvDll();
AnsiString __fastcall GPGIdeaDll();
AnsiString __fastcall GPGDefaultKeyID();
bool __fastcall GPGIsPGPMessage(TStrings *Message);
bool __fastcall GPGIsPGPSignedMessage(TStrings *Message);
bool __fastcall GPGHasPGPSignatureVerification(TStrings *Message);

bool __fastcall GPGGetEncryptionKeys(TStrings *Message, TStrings *Keys);
bool __fastcall GPGDecryptMessage(TForm *Owner, TStrings *Message, AnsiString &Passphrase,
   bool TryAll, bool Quiet=false);
//bool __fastcall GPGVerifyClearSigned(TForm *Owner, TStrings *Message);
bool __fastcall GPGEncryptStringToStream(AnsiString &Passphrase, AnsiString &Input, TStream *Output);
bool __fastcall GPGDecryptStreamToString(AnsiString &Passphrase, TStream *Input, AnsiString &Output);

bool __fastcall setfield(const AnsiString &field, const AnsiString &value, AnsiString &line);
AnsiString __fastcall getcommandline(const AnsiString &item, const AnsiString defvalue);
bool __fastcall checkgpgavailable();
void __fastcall getheaders(TStrings *headers, TStrings *message);
void __fastcall putheaders(TStrings *message, TStrings *headers);
void __fastcall FormatSigVerification(TStrings *verified);
//void __fastcall CheckFixQPInArmor(TStrings *message);

class TGPGConsole : public TObject
{
protected:
   TForm *Owner;
   bool running;
   STARTUPINFO startupinfo;

   void __fastcall PrepareStartupInfo();
   bool __fastcall CreateHandles();
   void __fastcall SetupFileDescriptors(AnsiString &cmdline);
   bool __fastcall LaunchChildProcess(const AnsiString &cmdline);
   void __fastcall DestroyHandles();
public:
   HANDLE hOutputRead, hOutputWrite, hInputRead, hInputWrite, hPassRead,
	  hPassWrite, hErrorRead, hErrorWrite, hAttrRead, hAttrWrite,
	  hStatRead, hStatWrite;
   PROCESS_INFORMATION processinfo;
   TMemoryStream *errorstream, *statusstream;
   TStringList *error, *status, *attrib;
   DWORD exitcode, exitwait;
   bool maskversion, bstatus, battrib, bpasswd;

   __fastcall TGPGConsole(TForm *Owner);
   __fastcall virtual ~TGPGConsole();
   bool __fastcall Run(AnsiString cmdLine, char *Passphrase, TStrings *Input, TStrings *Output);
   bool __fastcall Run(AnsiString cmdLine, AnsiString &Passphrase, const AnsiString &Input, TStream *Output);
   bool __fastcall Run(AnsiString cmdLine, AnsiString &Passphrase, TStream *Input, AnsiString &Output);
   bool __fastcall SendInput(TStream *input);
   bool __fastcall SendInput(const AnsiString &input);
   bool __fastcall SendInput(TStrings *input);
   bool __fastcall ReadOutput(TStrings *output);
   bool __fastcall ReadOutput(AnsiString &output);
   bool __fastcall ReadOutput(TStream *output);
   bool __fastcall GetWaiting(HANDLE hReadHandle, TStrings *output);
   bool __fastcall GetWaiting(HANDLE hReadHandle, AnsiString &output);
   bool __fastcall GetWaiting(HANDLE hReadHandle, TStream *output);
   void __fastcall ShowError(const AnsiString &errormsg);
   bool __fastcall SendPassphrase(char *passphrase);
   void __fastcall Terminate();
};

class TGPGerrDlg : public TForm
{
__published:	// IDE-managed Components
		TLabel *Label1;
		TRichEdit *ErrorBox;
		TButton *StatusBtn;
		TButton *OkBtn;
		TPopupMenu *PopupMenu1;
		TMenuItem *Copy1;
	TMenuItem *Selectall1;
		void __fastcall FormShow(TObject *Sender);
		void __fastcall StatusBtnClick(TObject *Sender);
		void __fastcall PopupMenu1Popup(TObject *Sender);
		void __fastcall Copy1Click(TObject *Sender);
	void __fastcall Selectall1Click(TObject *Sender);
private:	// User declarations
public:		// User declarations
		TStrings *status, *error;
		__fastcall virtual TGPGerrDlg(TComponent* Owner);
		__fastcall virtual TGPGerrDlg(TComponent* Owner, TStrings *GPGerror,
				TStrings *GPGstatus);
		__fastcall virtual ~TGPGerrDlg();
};

const AnsiString BeginREMAILERMessage = "-----BEGIN REMAILER MESSAGE-----";
const AnsiString BeginPGPMessage = "-----BEGIN PGP MESSAGE-----";
const AnsiString EndPGPMessage = "-----END PGP MESSAGE-----";
const AnsiString BeginPGPSignedMessage = "-----BEGIN PGP SIGNED MESSAGE-----";
const AnsiString BeginPGPSignature = "-----BEGIN PGP SIGNATURE-----";
const AnsiString EndPGPSignature = "-----END PGP SIGNATURE-----";
const AnsiString BeginPGPDecryptedVerifiedMessage = "*** BEGIN PGP DECRYPTED/VERIFIED MESSAGE ***";
const AnsiString EndPGPDecryptedVerifiedMessage = "*** END PGP DECRYPTED/VERIFIED MESSAGE ***";
const AnsiString BeginPGPDecryptedMessage = "*** BEGIN PGP DECRYPTED MESSAGE ***";
const AnsiString EndPGPDecryptedMessage = "*** END PGP DECRYPTED MESSAGE ***";
const AnsiString EndPGPVerifiedMessage = "*** END PGP VERIFIED MESSAGE ***";
const AnsiString PGPSignatureVerification = "*** PGP SIGNATURE VERIFICATION ***";
const AnsiString BeginPGPPublicKeyBlock = "*** BEGIN PGP PUBLIC KEY BLOCK ***";
const AnsiString EndPGPPublicKeyBlock = "*** END PGP PUBLIC KEY BLOCK ***";
const AnsiString BeginPGPArmoredFile = "-----BEGIN PGP ARMORED FILE-----";
const AnsiString EndPGPArmoredFile = "-----END PGP ARMORED FILE-----";

const AnsiString GPGDecryptDBCmd =
   " --status-fd %stat% --batch --no-tty --yes --passphrase-fd %pass% -d -v %filename%";
const AnsiString GPGDecryptCmd =
   " --status-fd %stat% --batch --no-tty --yes --passphrase-fd %pass% -v -d --verify-options show-primary-uid-only";
const AnsiString GPGDecryptAllCmd =
   " --status-fd %stat% --batch --no-tty --yes --passphrase-fd %pass% -d --verify-options show-primary-uid-only";
const AnsiString GPGEncryptDBCmd =
   " --status-fd %stat% --batch --no-tty --passphrase-fd %pass% --yes -c --output %filename%";
const AnsiString GPGVerifyClearSignedCmd =
   " --status-fd %stat% -d --verify-options show-primary-uid-only";
const AnsiString GPGListEncryptionKeysCmd =
   " --status-fd %stat% --batch -v -v --decrypt --list-only";
const AnsiString GPGListEncryptionKeysFromFileCmd =
   " --status-fd %stat% --batch -v -v --decrypt --list-only %filename%";
const AnsiString GPGEncryptStringToStreamCmd =
   " --status-fd %stat% --batch --no-tty --passphrase-fd %pass% -c";
const AnsiString GPGDecryptStreamToStringCmd =
   " --status-fd %stat% --batch --no-tty --passphrase-fd %pass% -d";
const AnsiString GPGCEncryptToFileNACmd =
   " --status-fd %stat% --batch --no-tty --passphrase-fd %pass% --yes -c --output %filename%";
const AnsiString GPGDecryptFromFileCmd =
   " --status-fd %stat% --batch --no-tty --passphrase-fd %pass% -d --verify-options show-primary-uid-only %filename%";
const AnsiString GPGListSecretKeysWithColonsCmd =
   " --status-fd %stat% --batch --list-secret-keys --with-colons --with-fingerprint";
const AnsiString GPGListPublicKeysWithColonsCmd =
   " --status-fd %stat% --batch --list-public-keys --with-colons --with-fingerprint";
const AnsiString GPGExportKeysCmd =
   " --status-fd %stat% --batch %armor% --export %keyids%";

// ini items of [gpg] section for replacement commandlines, etc
const AnsiString GPGlistencryptionkeys = "listencryptionkeys";
const AnsiString GPGlistencryptionkeysfromfile = "listencryptionkeysfromfile";
const AnsiString GPGverifyclearsigned = "verifyclear";
const AnsiString GPGdecryptall = "decryptall";
const AnsiString GPGdecrypt = "decrypt";
const AnsiString GPGdecryptdb = "decryptdb";
const AnsiString GPGdecryptfromfile = "decryptfromfile";
const AnsiString GPGencryptstringtostream = "encryptstringtostream";
const AnsiString GPGdecryptstreamtostring = "decryptstreamtostring";
const AnsiString GPGcencrypttofilena = "cencrypttofilena";
const AnsiString GPGencryptdb = "encryptdb";
const AnsiString GPGlistsecretkeyswithcolons = "listsecretkeyswithcolons";
const AnsiString GPGlistpublickeyswithcolons = "listpublickeyswithcolons";
const AnsiString GPGexportkeys = "exportkeys";

//const int GPGMinPassLength = 6;

// listkey fields for gpglistkeyswithcolons
const int lkfKeyType = 0;
const int lkfKeyValid = 1;
const int lkfKeyBits = 2;
const int lkfKeyAlgo = 3;
const int lkfKeyID = 4;
const int lkfKeyCreated = 5;
const int lkfKeyExpires = 6;
const int lkfKeyUID = 9;

void __fastcall ShowGPGError(TForm *Owner, TStrings *Error, TStrings *Status);
void __fastcall ShowGPGError(TForm *Owner, TStrings *ErrorStatus);
//---------------------------------------------------------------------------
extern PACKAGE TGPGerrDlg *GPGerrDlg;
extern AnsiString gpgdatadir;
extern AnsiString gpgdir;
//---------------------------------------------------------------------------
#endif
