/*
*  gpg2.h 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#ifndef gpg2H
#define gpg2H
#include <syncobjs.hpp>
class TStdIOInputThread;
class TStdIOOutputThread;
//---------------------------------------------------------------------------

bool __fastcall GPGProcessMessageDecryptionQuiet(TForm *Owner, TStrings *Message,
   AnsiString &Passphrase);

bool __fastcall GPGDecryptMessage3(TForm *Owner, TStrings *Message,
   AnsiString &Passphrase, bool Quiet, TStrings *Feedback = NULL);
bool __fastcall GPGGetEncryptionKeys3(TStrings *Message, TStrings *Keys);

bool __fastcall GPGClearSignMessage2(TForm *Owner, TStrings *Message,
   const AnsiString &SignKey, AnsiString &Passphrase, bool MaskVersion);
bool __fastcall GPGVerifyClearSigned2(TForm *Owner, TStrings *Message);
bool __fastcall GPGDecryptMessage2(TForm *Owner, TStrings *Message,
   AnsiString &Passphrase, bool Quiet, TStrings *Feedback = NULL);
bool __fastcall GPGGetEncryptionKeys2(TStrings *Message, TStrings *Keys);

class TGPGConsole2 : public TObject
{
protected:
   TForm *Owner;
   bool running;
   STARTUPINFO startupinfo;

   void __fastcall PrepareStartupInfo();
   bool __fastcall CreateHandles(const AnsiString &cmdLine);
   void __fastcall SetupFileDescriptors(AnsiString &cmdline);
   bool __fastcall LaunchChildProcess(const AnsiString &cmdline);
   void __fastcall DestroyHandles();
public:
   HANDLE hOutputRead, hOutputWrite, hInputRead, hInputWrite, hPassRead,
	  hPassWrite, hErrorRead, hErrorWrite, hAttrRead, hAttrWrite,
	  hStatRead, hStatWrite;
   PROCESS_INFORMATION processinfo;
   TStream *inputstream, *outputstream;
   TMemoryStream *errorstream, *statusstream;
   TStringList *error, *status, *attrib;
   TStdIOOutputThread *outputthread;
   TStdIOInputThread *inputthread;
   TEvent *inputevent;
   DWORD exitcode;

   __fastcall TGPGConsole2(TForm *Owner);
   __fastcall virtual ~TGPGConsole2();
   bool __fastcall Run(AnsiString cmdLine, char *Passphrase, TStream *Input, TStream *Output);
   void __fastcall ShowError(const AnsiString &errormsg);
   bool __fastcall SendPassphrase(char *passphrase);
   void __fastcall Terminate();
};

class TStdIOInputThread : public TThread
{
protected:
	void __fastcall Execute();
public:
   TGPGConsole2 *con;
   __fastcall TStdIOInputThread(TGPGConsole2 *Console);
};

class TStdIOOutputThread : public TThread
{
protected:
	void __fastcall Execute();
public:
   TGPGConsole2 *con;
   __fastcall TStdIOOutputThread(TGPGConsole2 *Console);
};

class TGPGConsole3 : public TObject
{
protected:
   TForm *Owner;
   bool running;
   STARTUPINFO startupinfo;

   void __fastcall PrepareStartupInfo();
   bool __fastcall CreateHandles(const AnsiString &cmdLine);
   void __fastcall SetupFileDescriptors(AnsiString &cmdline);
   void __fastcall SetupFileNames(AnsiString &cmdline);
   bool __fastcall LaunchChildProcess(const AnsiString &cmdline);
   void __fastcall DestroyHandles();
public:
   HANDLE hOutputRead, hOutputWrite, hInputRead, hInputWrite, hPassRead,
	  hPassWrite, hErrorRead, hErrorWrite, hAttrRead, hAttrWrite,
	  hStatRead, hStatWrite;
   AnsiString stafile,logfile,outfile,inpfile,datadir;
   PROCESS_INFORMATION processinfo;
   TStringList *error, *status, *logger;
   TEvent *inputevent;
   DWORD exitcode;

   __fastcall TGPGConsole3(TForm *Owner);
   __fastcall virtual ~TGPGConsole3();
   bool __fastcall Run(AnsiString cmdLine, char *Passphrase, TStrings *Input, TStrings *Output);
   void __fastcall ShowError(const AnsiString &errormsg);
   bool __fastcall SendPassphrase(char *passphrase);
   void __fastcall Terminate();
};
//---------------------------------------------------------------------------
#endif
