/*
*  pgpcache.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "PgpCache.h"
#include "qsadbs.h"
#include "gpg2.h"
#include "gnupg.h"
#include "utility.h"
#include "global.h"
//---------------------------------------------------------------------------
TPGPPassphraseCache *PGPCache;
const DWORD MaxDWORD = 4294967295;
//---------------------------------------------------------------------------
// TPhrase
//---------------------------------------------------------------------------

__fastcall TPGPPassphrase::TPGPPassphrase(const AnsiString &PGPIDText8,
   AnsiString &Passphrase, int Minutes) : TObject()
{
   AnsiString key;
   DB->LoadSessionKey(key);
   encryptedphrase = new TMemoryStream();
   GPGEncryptStringToStream(key, Passphrase, encryptedphrase);
   idtext8 = PGPIDText8;
   SetExpire(Minutes);
}
//---------------------------------------------------------------------------

__fastcall TPGPPassphrase::~TPGPPassphrase()
{
   ZeroMemory(encryptedphrase->Memory, (SIZE_T)encryptedphrase->Size);
   delete encryptedphrase;
}
//---------------------------------------------------------------------------

void __fastcall TPGPPassphrase::SetExpire(int minutes)
{
   starttick = GetTickCount();
   double expiretick = starttick + (minutes * 60000);
   lasttick = (DWORD)(expiretick > MaxDWORD ? expiretick - MaxDWORD : expiretick);
}
//---------------------------------------------------------------------------

bool __fastcall TPGPPassphrase::Expired()
{
   DWORD curtick = GetTickCount();
   return (lasttick <= curtick) && ((lasttick > starttick) || (curtick < starttick));
}

//---------------------------------------------------------------------------
// TPGPPassphraseCache
//---------------------------------------------------------------------------

__fastcall TPGPPassphraseCache::TPGPPassphraseCache()
{
   TIniFile *ini = new TIniFile(AppIni());
   minutes = ini->ReadInteger("settings","cachemins", 0);
   delete ini;

   timer = new TTimer(NULL);
   timer->Enabled = false;
   timer->Interval = 10000;  // 10 seconds
   timer->OnTimer = OnTimer;
   passphrases = new TList();
}
//---------------------------------------------------------------------------

__fastcall TPGPPassphraseCache::~TPGPPassphraseCache()
{
   Clear();
   for (int i = 0; i < passphrases->Count; i++)
   {
	  delete passphrases->Items[i];
	  passphrases->Items[i] = NULL;
	  passphrases->Delete(i);
   }
   delete timer;
}
//---------------------------------------------------------------------------

/*__fastcall TPGPPassphraseCache::~TPGPPassphraseCache()
{
   Clear();
   for (int i = 0; i < passphrases->Count; i++)
   {
	  TPGPPassphrase *phrase = (TPGPPassphrase*)passphrases->Items[i];
	  delete phrase;
	  phrase = NULL;
	  passphrases->Delete(i);
   }
   delete timer;
}*/
//---------------------------------------------------------------------------

void __fastcall TPGPPassphraseCache::Add(const AnsiString &idtext8, AnsiString &passphrase)
{
   assert(idtext8 != "");
   assert(passphrase != "");
   if (minutes == 0)
   {
	  ZeroString(passphrase);
	  return;
   }
   timer->Enabled = false;

   AnsiString IdText8, Passphrase;
   int i = 0; while (i < passphrases->Count)
   {
	  Get(i, IdText8, Passphrase);
	  if ((idtext8.Pos("00000000") >= 0 && IdText8 == idtext8) && (Passphrase == passphrase))
	  {
		 passphrases->Delete(i);
		 continue;
	  }
	  if (idtext8.Pos("00000000") < 0 && IdText8 == idtext8)
	  {
		 passphrases->Delete(i);
		 break;
	  }
	  i++;
   }

   passphrases->Add(new TPGPPassphrase(idtext8, passphrase, minutes));
   ZeroString(passphrase);
   timer->Enabled = true;
}
//---------------------------------------------------------------------------

/*void __fastcall TPGPPassphraseCache::Add(const AnsiString &idtext8, AnsiString &passphrase)
{
   assert(idtext8 != "");
   assert(passphrase != "");
   if (minutes == 0)
   {
	  ZeroString(passphrase);
	  return;
   }
   timer->Enabled = false;

   AnsiString IdText8, Passphrase;
   int i = 0; while (i < passphrases->Count)
   {
	  Get(i, IdText8, Passphrase);
	  if ((idtext8.Pos("00000000") >= 0 && IdText8 == idtext8) && (Passphrase == passphrase))
	  {
		 passphrases->Delete(i);
		 continue;
	  }
	  if (idtext8.Pos("00000000") < 0 && IdText8 == idtext8)
	  {
		 passphrases->Delete(i);
		 break;
	  }
	  i++;
   }

   passphrases->Add(new TPGPPassphrase(idtext8, passphrase, minutes));
   ZeroString(passphrase);
   timer->Enabled = true;
}*/
//---------------------------------------------------------------------------

/*void __fastcall TPGPPassphraseCache::Add(const AnsiString &idtext8, AnsiString &passphrase)
{
   assert(idtext8 != "");
   assert(passphrase != "");
   if (minutes == 0)
   {
	  ZeroString(passphrase);
	  return;
   }
   timer->Enabled = false;

   if (idtext8.Pos("00000000") > 0)
	  passphrases->Add(new TPGPPassphrase(idtext8, passphrase, minutes));
   else
   {
	  for (int i = 0; i < passphrases->Count; i++)
	  {
		 TPGPPassphrase *phrase = GetPGPPassphrase(i);
		 AnsiString IDText8 = phrase->idtext8;
		 if (idtext8.Pos("00000000") < 0 && IDText8 == idtext8)
		 {
			delete passphrases->Items[i];
			break;
		 }
	  }
   }
   passphrases->Add(new TPGPPassphrase(idtext8, passphrase, minutes));
   ZeroString(passphrase);
   timer->Enabled = true;
}*/
//---------------------------------------------------------------------------

bool __fastcall TPGPPassphraseCache::Get(TStrings *idtext8s, AnsiString &idtext8,
   AnsiString &passphrase) // returns idtext8 and it's phrase
{
   bool got = false;
   timer->Enabled = false;
   for (int i = 0; i < passphrases->Count; i++)
   {
	  TPGPPassphrase *phrase = GetPGPPassphrase(i);
	  if (idtext8s->IndexOf(phrase->idtext8) >= 0)
	  {
		 AnsiString key;
		 DB->LoadSessionKey(key);
		 GPGDecryptStreamToString(key, phrase->encryptedphrase, passphrase);
		 phrase->SetExpire(minutes);//DB->GetConfigItem(PGPConfig, GPGcacheminutes, GPGCacheMinutes));
		 idtext8 = phrase->idtext8;
		 got = true;
		 break;
	  }
   }
   timer->Enabled = true;
   return got;
}
//---------------------------------------------------------------------------

bool __fastcall TPGPPassphraseCache::Get(const AnsiString &idtext8, AnsiString &passphrase)
{
   bool enabled = timer->Enabled;
   if (enabled)
	  timer->Enabled = false;

   passphrase = "";
   bool got = false;

   for (int i = 0; i < passphrases->Count; i++)
   {
	  TPGPPassphrase *phrase = GetPGPPassphrase(i);
	  if (phrase->idtext8 == idtext8)
	  {
		 AnsiString key;
		 DB->LoadSessionKey(key);
		 GPGDecryptStreamToString(key, phrase->encryptedphrase, passphrase);
		 phrase->SetExpire(minutes);
		 got = true;
		 break;
	  }
   }

   if (enabled)
	  timer->Enabled = true;
   return got;
}

//---------------------------------------------------------------------------
// passphrases functions
/*bool __fastcall TPGPPassphraseCache::Get(TStrings *idtext8s, TStrings *Passphrases)
{
   timer->Enabled = false;

   Passphrases->Clear();
   bool throwkeys = idtext8s->Text.Pos("00000000");

   for (int i = 0; i < passphrases->Count; i++)
   {
	  AnsiString passphrase;
	  TPGPPassphrase *phrase = GetPGPPassphrase(i);
	  if (idtext8s->IndexOf(phrase->idtext8) >= 0 || throwkeys)
	  {
		 AnsiString key;
		 DB->LoadSessionKey(key);
		 GPGDecryptStreamToString(key, phrase->encryptedphrase, passphrase);
		 Passphrases->Add(passphrase);
		 phrase->SetExpire(minutes);
		 if (throwkeys)
			continue;
		 break;
	  }
   }

   timer->Enabled = true;
   return passphrases->Count;
}*/
//---------------------------------------------------------------------------

/*bool __fastcall TPGPPassphraseCache::Get(const AnsiString &idtext8, TStrings *Passphrases)
{
   bool enabled = timer->Enabled;
   if (enabled)
	  timer->Enabled = false;

   Passphrases->Clear();
   AnsiString passphrase;
   bool throwkeys = idtext8.Pos("00000000") > 0;

   for (int i = 0; i < passphrases->Count; i++)
   {
	  TPGPPassphrase *phrase = GetPGPPassphrase(i);
	  if (phrase->idtext8 == idtext8 || throwkeys)
	  {
		 AnsiString key;
		 DB->LoadSessionKey(key);
		 GPGDecryptStreamToString(key, phrase->encryptedphrase, passphrase);
		 Passphrases->Add(passphrase);
		 ZeroString(passphrase.Unique());
		 ZeroString(key);
		 phrase->SetExpire(minutes);
		 if (throwkeys)
			continue;
		 break;
	  }
   }
   if (enabled)
	  timer->Enabled = true;

   return passphrases->Count;
}*/
//---------------------------------------------------------------------------

bool __fastcall TPGPPassphraseCache::Get(int Index, AnsiString &idtext8, AnsiString &passphrase)
{
   TPGPPassphrase *phrase = GetPGPPassphrase(Index);
   idtext8 = phrase->idtext8;

   AnsiString key;
   DB->LoadSessionKey(key);
   GPGDecryptStreamToString(key, phrase->encryptedphrase, passphrase);
   ZeroString(key);
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TPGPPassphraseCache::SetMinutes(int Minutes)
{
   minutes = Minutes;
}
//---------------------------------------------------------------------------

TPGPPassphrase* __fastcall TPGPPassphraseCache::GetPGPPassphrase(int index)
{
   assert(index >= 0);
   assert(index < passphrases->Count);
   return (TPGPPassphrase*)passphrases->Items[index];
}
//---------------------------------------------------------------------------

/*TPGPPassphrase* __fastcall TPGPPassphraseCache::GetPGPThrowKeyid(int index)
{
   assert(index >= 0);
   assert(index < throwkeyids->Count);
   return (TPGPPassphrase*)throwkeyids->Items[index];
}*/
//---------------------------------------------------------------------------

void __fastcall TPGPPassphraseCache::OnTimer(TObject *Sender)
{
   timer->Enabled = false;
   TPGPPassphrase *phrase;
   int i = 0; while (i < passphrases->Count)
   {
	  phrase = GetPGPPassphrase(i);
	  if (phrase->Expired())
	  {
		 delete phrase;
		 passphrases->Delete(i);
		 continue;
	  }
	  i++;
   }
   if (passphrases->Count)
	  timer->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TPGPPassphraseCache::Clear()
{
   timer->Enabled = false;
   while (passphrases->Count)
   {
	  delete GetPGPPassphrase(0);
	  passphrases->Delete(0);
   }
}
//---------------------------------------------------------------------------

int __fastcall TPGPPassphraseCache::Count()
{
   return passphrases->Count;
}
//---------------------------------------------------------------------------

