/*
*  qsadbs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#pragma hdrstop
#include "qsadbs.h"
#include "utility.h"
#include "esub.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
TQSADB *DB;
//---------------------------------------------------------------------------
// QSDB
//---------------------------------------------------------------------------

TQSADB* __fastcall OpenQSADB(const AnsiString &FileName)
{
   assert(FileName != "");
   TQSADB *DB = new TQSADB(FileName);
   if (DB->error)
   {
	  delete DB;
	  return NULL;
   }
   return DB;
}
//---------------------------------------------------------------------------

bool __fastcall CloseQSADB(TQSADB *DB)
{
   return CloseSecureDB(DB);
}

//---------------------------------------------------------------------------
// TQSADB
//---------------------------------------------------------------------------

__fastcall TQSADB::TQSADB(const AnsiString &FileName)
   : TSecureDB(FileName)
{
}
//---------------------------------------------------------------------------

__fastcall TQSADB::~TQSADB()
{
}


//---------------------------------------------------------------------------
// messages
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadMessageIndex(TStrings *Index)
{
   return GetText("inbox", Index);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadMessage(const AnsiString &MsgKey, TStrings *Msg)
{
   return GetText("inbox"+DBPS+MsgKey, Msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveMessage(const AnsiString &MsgKey, TStrings *Msg)
{
   return PutText("inbox"+DBPS+MsgKey, Msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::DelMessage(const AnsiString &MsgKey)
{
   return DelItem("inbox"+DBPS+MsgKey);
}

//---------------------------------------------------------------------------
// news account hosts
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadAccounts(TStrings *accounts)
{
   return GetText(Hosts, accounts);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveAccounts(TStrings *accounts)
{
   return PutText(Hosts, accounts);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadAccount(const AnsiString &name, TStrings *account)
{
   TStringList *accts = new TStringList();
   bool ok = LoadAccounts(accts);
   account->CommaText = accts->Values[name];
   delete accts;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveAccount(const AnsiString &name, TStrings *account)
{
   TStringList *accts = new TStringList();
   bool ok = LoadAccounts(accts);
   accts->Values[name] = account->CommaText;
   if (ok) ok = SaveAccounts(accts);
   delete accts;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::AccountExists(const AnsiString &name)
{
   TStringList *accts = new TStringList();
   bool ok = LoadAccounts(accts);
   if (ok)
	  ok = accts->Values[name] != "";
   delete accts;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::DelAccount(const AnsiString &name)
{
   TStringList *accts = new TStringList();
   bool ok = LoadAccounts(accts);
   if (ok)
   {
	  accts->Values[name] = "";
	  ok = SaveAccounts(accts);
   }
   delete accts;
   return ok;
}

//---------------------------------------------------------------------------
// proxies
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadProxies(TStrings *proxies)
{
   return GetText(Proxies, proxies);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveProxies(TStrings *proxies)
{
   return PutText(Proxies, proxies);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadProxy(const AnsiString &name, TStrings *proxy)
{
   TStringList *proxies = new TStringList();
   bool ok = LoadProxies(proxies);
   proxy->CommaText = proxies->Values[name];
   delete proxies;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveProxy(const AnsiString &name, TStrings *proxy)
{
   TStringList *proxies = new TStringList();
   bool ok = LoadProxies(proxies);
   proxies->Values[name] = proxy->CommaText;
   if (ok) ok = SaveProxies(proxies);
   delete proxies;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::ProxyExists(const AnsiString &name)
{
   TStringList *proxies = new TStringList();
   bool ok = LoadProxies(proxies);
   if (ok)
	  proxies->Values[name] != "";
   delete proxies;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::DelProxy(const AnsiString &name)
{
   TStringList *proxies = new TStringList();
   bool ok = GetText(Proxies, proxies);
   if (ok)
   {
	  proxies->Values[name] = "";
	  ok = PutText(Proxies, proxies);
   }
   delete proxies;
   return ok;
}

//---------------------------------------------------------------------------
// subjects
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadSubjects(TStrings *subjects)
{
   return GetText("subjects", subjects);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveSubjects(TStrings *subjects)
{
   return PutText("subjects", subjects);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadSubject(const AnsiString &subno, TStrings *subject)
{
   subject->Clear();
   TStringList *sub, *subs = new TStringList();
   bool ok = GetText("subjects", subs);

   if (ok)
   {
	  sub = new TStringList();
	  for (int i = 0; i < subs->Count; i++)
	  {
		 sub->CommaText = subs->Strings[i];
		 if (sub->Values["subno"] == subno)
		 {
			subject->AddStrings(sub);
			break;
		 }
	  }
   }

   delete sub;
   delete subs;
   return ok;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSADB::LoadSubject(const AnsiString &subno)
{
   assert(subno != "");
   AnsiString subject = "";
   TStringList *sub, *subs = new TStringList();
   bool ok = GetText("subjects", subs);
   if (ok)
   {
	  sub = new TStringList();
	  for (int i = 0; i < subs->Count; i++)
	  {
		 sub->CommaText = subs->Strings[i];
		 if (sub->Values["subno"] == subno)
		 {
			subject = subs->Strings[i];
			break;
		 }
	  }
   }

   delete sub;
   delete subs;
   return subject;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveSubject(const AnsiString &subno, TStrings *subject)
{
   assert(subno != "");
   assert(subject != NULL && subject->Count);
   assert(subject->Values["subno"] != "");

   TStringList *sub, *subs = new TStringList();
   bool saved = false, ok = GetText("subjects", subs);
   if (ok)
   {
	  sub = new TStringList();
	  for (int i = 0; i < subs->Count; i++)
	  {
		 sub->CommaText = subs->Strings[i];
		 if (sub->Values["subno"] == subno)
		 {
			subs->Strings[i] = subject->CommaText;
			saved = true;
			break;
		 }
	  }
	  if (!saved)
		 subs->Add(subject->CommaText);
	  ok = PutText("subjects", subs);
   }
   delete sub;
   delete subs;
   return ok;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSADB::GetSubjectItem(const AnsiString &subno, const AnsiString &item,
   const AnsiString &Default)
{
   AnsiString itemstr;
   TStringList *sub = new TStringList(), *subs = new TStringList();
   LoadSubjects(subs);
   for (int i = 0; i < subs->Count; i++)
   {
	  sub->CommaText = subs->Strings[i];
	  if (sub->Values["subno"] == subno)
	  {
		 itemstr = sub->Values[item];
		 break;
	  }
   }
   delete subs;
   delete sub;
   return itemstr;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SubjectExists(const AnsiString &Subject)
{
   TStringList *subs = new TStringList();
   GetText("subjects", subs);
   bool ok = subs->Values[Subject] != "";
   delete subs;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::DelSubject(const AnsiString &Subject)
{
   TStringList *subs = new TStringList();
   bool ok = GetText("subjects", subs);
   if (ok)
   {
	  subs->Values[Subject] = "";
	  ok = PutText("subjects", subs);
   }
   delete subs;
   return ok;
}

//---------------------------------------------------------------------------
// nymservers
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadNymservers(TStrings *servers)
{
   return GetText("nymservers", servers);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSADB::GetNymserverType(const AnsiString &name)
{
   TStringList *server = new TStringList();
   TStringList *servers = new TStringList();
   LoadNymservers(servers);
   server->CommaText = servers->Values[name];
   AnsiString type = server->Values["type"];
   if (type.IsEmpty())
	 type = "error";
   //assert(type != "");
   delete servers;
   delete server;
   return type;
}
//---------------------------------------------------------------------------

/*AnsiString __fastcall TQSADB::GetNymserverType(const AnsiString &server)
{
   TStringList *servers = new TStringList();
   LoadNymservers(servers);
   AnsiString type = servers->Values[server];
   if (type.IsEmpty())
	 type = "error";
   //assert(type != "");
   delete servers;
   return type;
}*/
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadNymserverDomains(TStrings *domains)
{
   domains->Clear();
   TStringList *servers = new TStringList();
   bool ok = GetText("nymservers", servers);
   if (ok)
   {
	  for (int i = 0; i < servers->Count; i++)
	  {
		 AnsiString domain = servers->Names[i];
		 domains->AddObject(domain, (TObject*)(servers->Values[domain].LowerCase() == "ghio" ? 1 : 0));
	  }
   }
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveNymserver(const AnsiString &name, TStrings *server)
{
   TStringList *servers = new TStringList();
   bool ok = LoadNymservers(servers);
   if (ok)
   {
	  servers->Values[name] = server->CommaText;
	  ok = SaveNymservers(servers);
   }
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadNymserver(const AnsiString &name, TStrings *server)
{
   server->Clear();
   TStringList *servers = new TStringList();
   bool ok = LoadNymservers(servers);
   if (ok)
	  server->CommaText = servers->Values[name];
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveNymservers(TStrings *servers)
{
   return PutText("nymservers", servers);
}

//---------------------------------------------------------------------------
// status
//---------------------------------------------------------------------------

bool __fastcall TQSADB::SaveStatus(TStrings *Status)
{
   return PutText("status", Status);
}
//---------------------------------------------------------------------------

bool __fastcall TQSADB::LoadStatus(TStrings *Status)
{
   return GetText("status", Status);
}
//---------------------------------------------------------------------------
